%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_p holds the piece-wise linear solution for
%  VariableName.  VariableName_l holds the linear solution for
%  VariableName.

setpathdynare4
global M_ oo_

% modname below chooses model
% directory. But simple param choices are made from paramfile in current
% directory.
modnam = 'cgg';
modnamstar = 'cgg_zlb';


% express the occasionally binding constraint
% in linearized form
% one can use any combination of endogenous variables and parameters 
% declared in the the dynare .mod files
constraint = 'r<(1-1/BETA)'; 
constraint_relax ='rnot>(1-1/BETA)';


% Pick innovation for IRFs
irfshock =char('eps_p','eps_r','eps_y');      % label for innovation for IRFs
                             % needs to be an exogenous variable in the
                             % dynare .mod files


                             
T = 100;

rng default

STD_P=0.001*1;
STD_R=0.001*1;
STD_Y=0.02*1;
                             
sequence = [ STD_P*randn(T,1) STD_R*randn(T,1) STD_Y*randn(T,1)  ];         % scale factor for simulations
nperiods = size(sequence,1);            %length of IRFs

LAMBDA=0.04;
BETA=0.9925;
FIP=4;
FIY=4;
FIR=0.8;
PHI=1;
RHOY=0.5;
save PARAM_EXTRA_BABY LAMBDA BETA FIP FIY FIR PHI RHOY STD_P STD_R STD_Y
  


% generate model IRFs
[zdatalinear zdatapiecewise zdatass oobase_ Mbase_  ] = ...
  solve_one_constraint(modnam,modnamstar,...
  constraint, constraint_relax,...
  sequence,irfshock,nperiods);


                          
% unpack the IRFs                          
for i=1:Mbase_.endo_nbr
  eval([deblank(Mbase_.endo_names(i,:)),'_l=zdatalinear(:,i);']);
  eval([deblank(Mbase_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
  eval([deblank(Mbase_.endo_names(i,:)),'_ss=zdatass(i);']);
end

% Construct interest rate in levels
rlevel_ss=1/BETA-1;
rlevel_p=400*(r_p+rlevel_ss);
rlevel_l=400*(r_l+rlevel_ss);
rlonglevel_ss=1/BETA-1;
rlonglevel_p=400*(rlong_p+rlonglevel_ss);
rlonglevel_l=400*(rlong_l+rlonglevel_ss);



%% Modify to plot IRFs and decision rules

titlelist = char('r (Interest Rate)','rlong','p (Inflation)','y (Output)','aap','aar','aay');


figtitle = '';
line1=[rlevel_p,rlonglevel_p,p_p*400,y_p*100,rlonglevel_p,aap_p*100,aar_p*100,aay_p*100];
line2=[rlevel_l,rlonglevel_l,p_l*400,y_l*100,rlonglevel_p,aap_l*100,aar_l*100,aay_l*100];


% Figure 1: Plot dynamic simulation
legendlist = cellstr(char('Piecewise Linear','Linear'));
ylabels = char('Percent Level, Annualized','Percent Level, Annualized','Percent from ss, Annualized','Percent from ss','Percent from ss','Percent from ss','Percent from ss');
figlabel = '';
makechart(titlelist,legendlist,figlabel,ylabels,line1,line2)


% There are no monetary shock at ZLB, update sequence accordingly
sequence = [ sequence(:,1) aar_p sequence(:,3) ];
save fakedata aap_p aar_p aay_p p_p r_p rlong_p rnot_p y_p sequence 

